
%------Simulation Parameters---------------%
iters = 1000; %number of iterations
nRS = 90; % number of cells of type RS
nFS = 30; %number of cells of type FS
%------------------------------------------%

%-------Connectivity Matrices--------------%
% C{1,1} = zeros(nRS,nRS); %RS-RS
C{1,1} = diag(ones(nRS-1,1),1);%The RS's are connected along a chain                              
% C{1,1} = rand(nRS,nRS); %RS-RS
% C{1,1} = C{1,1}>0.5;
C{2,1} = rand(nFS,nRS); %FS-RS
C{2,1} = C{2,1}>0.5;
C{1,2} = rand(nRS,nFS); %RS-FS
C{1,2} = C{1,2}>0.5;
C{2,2} = rand(nFS,nFS); %RS-FS
C{2,2} = C{2,2}>0.5;

%------------------------------------------%

%external input
RS.extIP = zeros(iters,nRS); 
RS.extIP(10:500,1) = 0.1; %external input to RS cells
                          %a pulse of amplitude 0.1 applied between
                          %iterations 10 to 500
FS.extIP = zeros(iters,nFS);
FS.extIP(100:300,1) = 0; %external input to FS cells

%---RS and FS cell properties--------------%
RS.param.beta_e = 0.133;
RS.param.beta_ext = 0.133; 
RS.param.alp = 3.65;
RS.param.mu = 0.0005;
RS.param.sig = 0.06;
RS.param.sig_e = 1.0;
RS.param.sig_ext = 1.0;

FS.param.beta_e = 0.1;
FS.param.beta_hp = 0.5;
FS.param.beta_ext = 0.1
FS.param.alp = 3.8;
FS.param.mu = 0.002;
FS.param.sig = 0.06;
FS.param.sig_e = 1.0;
FS.param.sig_ext = 1.0;

%------------------------------------------%

%------Synaptic conductances----------------%
% 1) AMPA synapse
g_AMPA = 0.85; %g_syn
AMPA_gam = 0.6; %gamma
AMPA_xrp = 0.0; %reversal potential

% 2) GABA_A 
g_GABA_A = 0.0; %g_syn
GABA_gam = 0.6; %gamma
GABA_x_rp = -1.1; %reversal potential

%--------------------------------------------%

%---Other ionic currents---------------------%
% 1) Hyperpolarizing current
%RS
g_hpRS = 0.1;
gamma_hpRS = 0.85;
beta_hpRS = 0.27;
y_rsRS = -2.9;

%FS
g_hpFS = 0.9;
gamma_hpFS = 0.5;
beta_hpFS = 0.5;
y_rsFS = -2.9;

%---------------------------------------------%

%--initial conditions-------------------------%
RS.x = -0.94*ones(iters,nRS);
RS.y = -2.82*ones(iters,nRS);
FS.x = -0.94*ones(iters,nFS);
FS.y = -2.82*ones(iters,nFS);
Isyn{1,1} = zeros(iters,nRS); %synaptic current from RS to RS
Isyn{2,1} = zeros(iters,nRS); %FS to RS

Isyn{1,2} = zeros(iters,nFS); %RS to FS
Isyn{2,2} = zeros(iters,nFS); %FS to FS

spikeRS_pre = zeros(1,nRS); %initialize spikes 
spikeFS_pre = zeros(1,nFS); %(spike_pre(ii) = 1 if cell ii fires, 
                            %               = 0 otherwise)

I_hpRS = zeros(iters,nRS); %hyperpolarizing current
I_hpFS = zeros(iters,nFS);
%----------------------------------------------%


for ii = 3:iters
    %-------------------synaptic inputs--------------------------------
    %synaptic input to RS cells-----------------------
    %RS-RS connections
    Isyn{1,1}(ii,:) = Isynaptic2(Isyn{1,1}(ii-1,:),... %synaptic ip in the previous step
        g_AMPA,... %AMPA synaptic conductivity
        AMPA_gam,...%gamma
        AMPA_xrp,...%synaptic reversal potential
        RS.x(ii-1,:),...%activity of the postsynaptic cells
        spikeRS_pre,...%which cells spiked in the last iteration
        C{1,1});%connectivity matrix

    %FS-RS connections
    Isyn{2,1}(ii,:) = Isynaptic2(Isyn{2,1}(ii-1,:),...
        g_GABA_A,...
        GABA_gam,...
        GABA_x_rp,...
        RS.x(ii-1,:),...
        spikeFS_pre,...
        C{2,1});
    
    %Synaptic input to FS cells-----------------------
    %FS-FS connections
    Isyn{2,2}(ii,:) = Isynaptic2(Isyn{2,2}(ii-1,:),...
        g_GABA_A,...
        GABA_gam,...
        GABA_x_rp,...
        FS.x(ii-1,:),...
        spikeFS_pre,...
        C{2,2});
    
    %RS-FS connections
    Isyn{1,2}(ii,:) = Isynaptic2(Isyn{1,2}(ii-1,:),...
        g_AMPA,...
        AMPA_gam,...
        AMPA_xrp,...
        FS.x(ii-1,:),...
        spikeRS_pre,...
        C{1,2});
    
    %hyperpolarizing current
    I_hpRS(ii,:) = I_hp(I_hpRS(ii-1,:),gamma_hpRS,g_hpRS,spikeRS_pre);
    I_hpFS(ii,:) = I_hp(I_hpFS(ii-1,:),gamma_hpFS,g_hpFS,spikeFS_pre);    
    %-----------------------------------------------------------------%

    %calculate the next iteration
    [RS.x(ii,:),RS.y(ii,:),uRS] = RS2(RS.x(ii-2:ii-1,:),RS.y(ii-2:ii-1,:),Isyn{1,1}(ii-1,:),Isyn{2,1}(ii-1,:),I_hpRS(ii-1,:),RS.extIP(ii-1,:),RS.param);
    [FS.x(ii,:),FS.y(ii,:),uFS] = FS2(FS.x(ii-2:ii-1,:),FS.y(ii-2:ii-1,:),Isyn{2,2}(ii-1,:),Isyn{1,2}(ii-1,:),I_hpFS(ii-1,:),FS.extIP(ii-1,:),FS.param);
    
    %conditions for spike_pre == 1
    spikeRS_pre = (RS.x(ii,:) >= uRS + RS.param.alp) | (RS.x(ii-1,:) > 0);
    spikeFS_pre = (FS.x(ii,:) >= uFS + FS.param.alp) | (FS.x(ii-1,:) > 0);
    
end

figure;
imagesc(RS.x); colorbar% plots the spatiotemporal activity of the RS cells
title('Activity of a chain of RS type neurons')
xlabel('Neuron index');
ylabel('time')